@@ -14,59 +14,43 @@ class Patch:
     :ivar extent: :class:`Extent` object containing 3-dimensional extent information.
     :ivar orientation: The direction the patch is facing (x={-1;1}, y={-2;2}, z={-3;3}).
     :ivar data: Numpy ndarray with the actual data.
-    :ivar t_n: Total number of time steps for which output data has been written.
+    :ivar n_t: Total number of time steps for which output data has been written.
     """
 
     def __init__(self, file_path: str, dimension: Dimension, extent: Extent, orientation: int,
-                 cell_centered: bool,
-                 initial_offset: int):
+                 cell_centered: bool, initial_offset: int, n_t: int):
         self.file_path = file_path
         self.dimension = dimension
         self.extent = extent
         self.orientation = orientation
         self.cell_centered = cell_centered
         self.initial_offset = initial_offset
-        # self.t_n = -1
         self.time_offset = -1
+        self.n_t = n_t
 
     @property
     def shape(self) -> Tuple:
         """Convenience function to calculate the shape of the array containing data for this patch.
         """
-        if abs(self.orientation) == 1:
-            dim = tuple(map(add, self.dimension.shape(self.cell_centered), (0, 2, 2)))
-        elif abs(self.orientation) == 2:
-            dim = tuple(map(add, self.dimension.shape(self.cell_centered), (2, 0, 2)))
-        else:
-            dim = tuple(map(add, self.dimension.shape(self.cell_centered), (2, 2, 0)))
-        return dim
+        return self.dimension.shape(self.cell_centered)
 
-    def _post_init(self, t_n: int, time_offset: int):
+    def _post_init(self, time_offset: int):
         """Fully initialize the patch as soon as the number of timesteps is known.
         """
         self.time_offset = time_offset
-        self.t_n = t_n
-
-    @property
-    def t_n(self):
-        return self._t_n
-
-    @t_n.setter
-    def t_n(self, t):
-        # print(self, t)
-        self._t_n = t
 
     @property
     def data(self):
         """Method to load the quantity data for a single patch for a single timestep.
         """
         if not hasattr(self, "_data"):
-            self._data = np.empty((self.t_n,) + self.shape)
-            dtype_data = np.dtype(fdtype.new_raw((('f', str(self.shape)),)))
+            self._data = np.empty((self.n_t,) + self.shape)
+            dtype_data = fdtype.new((('f', str(self.shape)),))
             with open(self.file_path, 'rb') as infile:
-                for t in range(self.t_n):
+                for t in range(self.n_t):
                     infile.seek(self.initial_offset + t * self.time_offset)
-                    self._data[t, :] = np.fromfile(infile, dtype_data, 1)
+                    self._data[t, :] = np.fromfile(infile, dtype_data, 1)[0][1].reshape(self.shape,
+                                                                                        order='F')
         return self._data
 
     def clear_cache(self):
@@ -80,11 +64,14 @@ class Patch:
 
 
 class Boundary:
-    def __init__(self, cell_centered: bool, quantity: Quantity, times: np.ndarray, t_n: int):
+    def __init__(self, cell_centered: bool, quantity: Quantity, times: np.ndarray, n_t: int,
+                 lower_bounds: np.ndarray, upper_bounds: np.ndarray):
         self.cell_centered = cell_centered
         self.quantity = quantity
         self.times = times
-        self.t_n = t_n
+        self.lower_bounds = lower_bounds
+        self.upper_bounds = upper_bounds
+        self.n_t = n_t
         self.extent = None
 
         self._patches: Dict[Mesh, List[Patch]] = dict()
@@ -100,7 +87,6 @@ class Boundary:
         if len(patches) != 0:
             patches_cart = [[patches[0]]]
             orientation = abs(patches[0].orientation)
-            # print(patches[0].orientation)
             if orientation == 1:  # x
                 patches.sort(key=lambda p: (p.extent.y_start, p.extent.z_start))
             elif orientation == 2:  # y
@@ -148,49 +134,36 @@ class Boundary:
         self._faces: Dict[Literal[-3, -2, -1, 1, 2, 3], np.ndarray] = dict()
         for face in (-3, -2, -1, 1, 2, 3):
             patches = self.sort_patches_cartesian(patches_for_face[face])
-            shape = [self.t_n, 0, 0]
-            for patch in patches:
-                if abs(face) == 1:
-                    shape[1] += patch.shape[1]
-                    shape[2] += patch.shape[2]
-                elif abs(face) == 2:
-                    shape[1] += patch.shape[0]
-                    shape[2] += patch.shape[2]
-                else:
-                    shape[1] += patch.shape[0]
-                    shape[2] += patch.shape[1]
-
-            self._faces[face] = np.ndarray(shape=shape)
+            if len(patches) == 0:
+                continue
 
+            shape_dim1 = sum([patch_row[0].shape[0] for patch_row in patches])
+            shape_dim2 = sum([patch.shape[1] for patch in patches[0]])
+            self._faces[face] = np.ndarray(shape=(self.n_t, shape_dim1, shape_dim2))
             dim1_pos = 0
             dim2_pos = 0
-            if abs(face) == 1:
-                dim1 = 1
-                dim2 = 2
-            elif abs(face) == 2:
-                dim1 = 0
-                dim2 = 2
-            else:
-                dim1 = 0
-                dim2 = 1
-            d1_temp = patches[0].shape[dim1]
-            d2_temp = patches[0].shape[dim2]
-
-            for patch in patches:
-                d1 = patch.shape[dim1] - d1_temp
-                d2 = patch.shape[dim2] - d2_temp
-                d1_temp = patch.shape[dim1]
-                d2_temp = patch.shape[dim2]
-                self._faces[face][:, dim1_pos:dim1_pos + d1, dim2_pos:dim2_pos + d2] = np.squeeze(
-                    patch.data)
+            for patch_row in patches:
+                d1 = patch_row[0].shape[0]
+                for patch in patch_row:
+                    d2 = patch.shape[1]
+                    self._faces[face][:, dim1_pos:dim1_pos + d1,
+                    dim2_pos:dim2_pos + d2] = patch.data
+                    dim2_pos += d2
                 dim1_pos += d1
-                dim2_pos += d2
+                dim2_pos = 0
+
+    def clear_cache(self):
+        """Remove all data from the internal cache that has been loaded so far to free memory.
+        """
+        for patches in self._patches.values():
+            for patch in patches:
+                patch.clear_cache()
 
     def __getitem__(self, item):
         if type(item) == Mesh:
             return self.get_patches_in_mesh(item)
         else:
-            return self.get_face(item)
+            return self.faces[item]
 
     def __repr__(self, *args, **kwargs):
         return f"Boundary(quantity={self.quantity}, cell_centered={self.cell_centered})"
@@ -240,17 +213,18 @@ class Obstruction:
     def _post_init(self):
         vals = self._extents.values()
         self.extent = Extent(
-            min(vals, key=lambda e: e.x_start).x_start, max(vals, key=lambda e: e.x_end).x_end, 0,
-            min(vals, key=lambda e: e.y_start).y_start, max(vals, key=lambda e: e.y_end).y_end, 0,
-            min(vals, key=lambda e: e.z_start).z_start, max(vals, key=lambda e: e.z_end).z_end, 0)
+            min(vals, key=lambda e: e.x_start).x_start, max(vals, key=lambda e: e.x_end).x_end,
+            min(vals, key=lambda e: e.y_start).y_start, max(vals, key=lambda e: e.y_end).y_end,
+            min(vals, key=lambda e: e.z_start).z_start, max(vals, key=lambda e: e.z_end).z_end)
         for boundary in self._boundary_data.values():
             boundary.extent = self.extent
 
     def _add_patches(self, bid: int, cell_centered: bool, quantity: str, label: str, unit: str,
-                     mesh: Mesh, patches: List[Patch], times: np.ndarray, t_n: int):
+                     mesh: Mesh, patches: List[Patch], times: np.ndarray, n_t: int,
+                     lower_bounds: np.ndarray, upper_bounds: np.ndarray):
         if bid not in self._boundary_data:
             self._boundary_data[bid] = Boundary(cell_centered, Quantity(quantity, label, unit),
-                                                times, t_n)
+                                                times, n_t, lower_bounds, upper_bounds)
         self._boundary_data[bid]._add_patches(mesh, patches)
 
         if not settings.LAZY_LOAD:
@@ -272,6 +246,12 @@ class Obstruction:
     def has_boundary_data(self):
         return len(self._boundary_data) != 0
 
+    def clear_cache(self):
+        """Remove all data from the internal cache that has been loaded so far to free memory.
+        """
+        for bndf in self._boundary_data.values():
+            bndf.clear_cache()
+
     def __getitem__(self, item):
         if type(item) == Quantity or type(item) == str:
             return self.get_boundary_data(item)
