@@ -1,9 +1,10 @@
 import os
-from typing import BinaryIO, List
+from typing import BinaryIO, Dict
 
 import numpy as np
 
-from fdsreader.utils import Quantity, settings, Mesh
+from fdsreader.utils import Quantity, Mesh
+from fdsreader import settings
 import fdsreader.utils.fortran_data as fdtype
 
 
@@ -15,7 +16,7 @@ class SubSurface:
     :ivar v_file_path: Path to the binary data file containing color data.
     :ivar n_vertices: The number of vertices for this subsurface.
     :ivar n_triangles: The number of triangles for this subsurface.
-    :ivar t_n: Total number of time steps for which output data has been written.
+    :ivar n_t: Total number of time steps for which output data has been written.
     :ivar _offset: Offset of the binary file to the end of the file header.
     """
 
@@ -120,6 +121,7 @@ class SubSurface:
                 dtype_triangles = fdtype.new((('i', 3 * n_triangles),))
                 dtype_surfaces = fdtype.new((('i', n_triangles),))
 
+                # Todo: Check for "order='F'"
                 vertices.append(
                     fdtype.read(infile, dtype_vertices, 1)[0][0].reshape((n_vertices, 3)))
                 triangles.append(
@@ -135,21 +137,35 @@ class SubSurface:
         self._triangles = np.array(triangles, dtype=object)
         self._surfaces = np.array(surfaces, dtype=object)
 
-        self.t_n = len(self.times)
+        self.n_t = len(self.times)
 
     def _load_vdata(self, infile: BinaryIO):
         """Loads all color data for all isosurfaces in a given viso file.
         """
-        self._colors = np.empty((self.t_n,), dtype=object)
+        self._colors = np.empty((self.n_t,), dtype=object)
         t_offset = fdtype.FLOAT.itemsize
         dtype_nverts = fdtype.new((('i', 4),)).itemsize
 
         infile.seek(fdtype.INT.itemsize * 2)
-        for t in range(self.t_n):
+        for t in range(self.n_t):
             infile.seek(t_offset, os.SEEK_CUR)
             n_vertices = fdtype.read(infile, dtype_nverts, 1)[0][0][2]
             self._colors[t] = fdtype.read(infile, fdtype.new((('f', n_vertices),)), 1)
 
+    def clear_cache(self):
+        """Remove all data from the internal cache that has been loaded so far to free memory.
+        """
+        if hasattr(self, "times"):
+            del self.times
+        if hasattr(self, "_vertices"):
+            del self._vertices
+        if hasattr(self, "_triangles"):
+            del self._triangles
+        if hasattr(self, "_surfaces"):
+            del self._surfaces
+        if hasattr(self, "_colors"):
+            del self._colors
+
 
 class Isosurface:
     """Isosurface file data container including metadata. Consists of a list of vertices forming a
@@ -172,7 +188,7 @@ class Isosurface:
 
         self._times = None
 
-        self._subsurfaces: List[SubSurface] = list()
+        self._subsurfaces: Dict[Mesh, SubSurface] = dict()
 
         if self._double_quantity:
             self.v_quantity = Quantity(v_quantity, v_label, v_unit)
@@ -183,17 +199,14 @@ class Isosurface:
                                     os.path.join(self.root_path, viso_filename))
         else:
             subsurface = SubSurface(mesh, os.path.join(self.root_path, iso_filename))
-        self._subsurfaces.append(subsurface)
+        self._subsurfaces[mesh] = subsurface
 
         return subsurface
 
-    def get_subsurface(self, mesh: Mesh):
+    def __getitem__(self, mesh: Mesh):
         """Returns the :class:`SubSurface` that contains data for the given mesh.
         """
-        for iso in self._subsurfaces:
-            if iso.mesh.id == mesh.id:
-                return iso
-        raise KeyError("The provided mesh is not valid for this operation in this simulation!")
+        return self._subsurfaces[mesh]
 
     @property
     def has_color_data(self):
@@ -207,3 +220,9 @@ class Isosurface:
             # Implicitly load the data for one subsurface and read times
             _ = self._subsurfaces[0].vertices
         return self._times
+
+    def clear_cache(self):
+        """Remove all data from the internal cache that has been loaded so far to free memory.
+        """
+        for subsurface in self._subsurfaces.values():
+            subsurface.clear_cache()
