@@ -1,16 +1,9 @@
-import logging
-
 import matplotlib.pyplot as plt
 
 import fdsreader as fds
 
 
 def main():
-    ch = logging.StreamHandler()
-    ch.setLevel(logging.DEBUG)
-
-    # fds.settings.ENABLE_CACHING = False
-
     sim = fds.Simulation("C:\\Users\\janv1\\PycharmProjects\\fdsreader\\examples\\slcf\\fds_steckler")
 
     mesh = sim.meshes[0]
@@ -25,10 +18,6 @@ def main():
 
     sslc_data = subslice.data
 
-    print(subslice)
-    # print(subslice.shape)
-    # print(sslc_data)
-
     plt.imshow(subslice.data[-1])
     plt.colorbar()
     plt.show()
