@@ -5,6 +5,8 @@ import glob
 import hashlib
 import os
 from collections import Iterable
+from typing import Union, Tuple
+import numpy as np
 
 
 class Quantity:
@@ -16,10 +18,32 @@ class Quantity:
     def __eq__(self, other):
         return self.quantity == other.quantity
 
+    def __hash__(self):
+        return hash(self.label)
+
     def __repr__(self):
         return f"Quantity(label={self.label}, unit={self.unit}, quantity={self.quantity})"
 
 
+class Device:
+    def __init__(self, quantity: Quantity, position: Tuple[float, float, float],
+                 orientation: Tuple[float, float, float]):
+        self.quantity = quantity
+        self.position = position
+        self.orientation = orientation
+        self.data: Union[float, np.ndarray] = .0
+
+    @property
+    def name(self):
+        return self.quantity.quantity
+
+    def __eq__(self, other):
+        return self.name == other.name
+
+    def __repr__(self):
+        return f"Device(name={self.name}, position={self.position}, quantity={self.quantity}, mean={np.mean(self.data)})"
+
+
 def create_hash(path: str):
     """Returns the md5 hash as string for the given file.
     """
@@ -51,6 +75,8 @@ def get_smv_file(path: str):
         elif len(files) == 0:
             raise IOError("There are no simulations in this path!")
         return files[0]
+    elif os.path.isfile(path + ".smv"):
+        return path + ".smv"
     else:
         raise IOError("Path is invalid!")
 
@@ -58,6 +84,7 @@ def get_smv_file(path: str):
 class FDSDataCollection:
     """(Abstract) Base class for any collection of FDS data.
     """
+
     def __init__(self, *elements: Iterable):
         self._elements = tuple(*elements)
 
@@ -73,6 +100,9 @@ class FDSDataCollection:
     def __contains__(self, value):
         return value in self._elements
 
+    def __repr__(self):
+        return "[" + str(self._elements) + "]"
+
     def clear_cache(self):
         """Remove all data from the internal cache that has been loaded so far to free memory.
         """
